import statistics.*;
import java.util.Random;

public class MM1Simulation {

  protected double lambda;
  protected double mu;

  public MM1Simulation(double lambda, double mu) {
    this.lambda = lambda;
    this.mu = mu;
  }
  
  public void simulate(double simLength) {
    Queue q = new Queue();
    FES fes = new FES();
    Random rand = new Random();
    Distribution interarrivalTimeDist = new ExponentialDistribution(lambda, rand);
    Distribution serviceTimeDist = new ExponentialDistribution(mu, rand);
    double t = 0;

    long beginTime = System.currentTimeMillis();
    // generate first arrival
    fes.addEvent(new Event(Event.ARRIVAL, interarrivalTimeDist.nextRandom()));
    
    while (t < simLength) {
      Event e = fes.nextEvent();
      t = e.getTime();
      switch (e.getType()) {
        case Event.ARRIVAL : {
          // schedule next arrival
          fes.addEvent(new Event(Event.ARRIVAL, t + interarrivalTimeDist.nextRandom()));

          double serviceTime = serviceTimeDist.nextRandom();
          q.addCustomer(new Customer(t, serviceTime));
          if (q.getSize() == 1) {
            fes.addEvent(new Event(Event.DEPARTURE, t + serviceTime));
          }
          break;
        }
        case Event.DEPARTURE : {
          q.removeCustomer(t);
          if (q.getSize() > 0) {
            double serviceTime = q.getCustomerAt(0).getServiceTime();
            fes.addEvent(new Event(Event.DEPARTURE, t + serviceTime));
          }
          break;
        }      
      }
    }
    System.out.println("Calculation time: "+(System.currentTimeMillis() - beginTime)*0.001+" seconds");
    System.out.println("Mean waiting time: "+q.getMeanWaitingTime());
    System.out.println("Var. waiting time: "+q.getVarianceWaitingTime());
    System.out.println("Mean sojourn time: "+q.getMeanSojournTime());
    System.out.println("Var. sojourn time: "+q.getVarianceSojournTime());
    System.out.println("Mean nr. of. customers: "+q.getMeanNumberOfCustomers());

  }
  
  public static void main(String[] arg) {
    MM1Simulation s = new MM1Simulation(0.5, 1.0);
    s.simulate(1000000);
  }
} 
